# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""The old client, which is deprecated, but uses the new client."""
from __future__ import annotations

import typing as t
import warnings

from polarion_rest_api_client import client
from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.client import WorkItemType


class OpenAPIPolarionProjectClient(t.Generic[WorkItemType]):
    """A Polarion Project Client using an auto generated OpenAPI-Client."""

    delete_status: str = "deleted"

    @t.overload
    def __init__(
        self: "OpenAPIPolarionProjectClient[client.WorkItemType]",
        project_id: str,
        delete_polarion_work_items: bool,
        polarion_api_endpoint: str,
        polarion_access_token: str,
        *,
        custom_work_item: type[client.WorkItemType],
        batch_size: int = ...,
        page_size: int = ...,
        add_work_item_checksum: bool = False,
        max_content_size: int = ...,
        httpx_args: t.Optional[dict[str, t.Any]] = ...,
    ): ...

    @t.overload
    def __init__(
        self: "OpenAPIPolarionProjectClient[dm.WorkItem]",
        project_id: str,
        delete_polarion_work_items: bool,
        polarion_api_endpoint: str,
        polarion_access_token: str,
        *,
        batch_size: int = ...,
        page_size: int = ...,
        add_work_item_checksum: bool = False,
        max_content_size: int = ...,
        httpx_args: t.Optional[dict[str, t.Any]] = ...,
    ): ...

    def __init__(
        self,
        project_id: str,
        delete_polarion_work_items: bool,
        polarion_api_endpoint: str,
        polarion_access_token: str,
        *,
        custom_work_item=dm.WorkItem,
        batch_size: int = 100,
        page_size: int = 100,
        add_work_item_checksum: bool = False,
        max_content_size: int = 2 * 1024**2,
        httpx_args: t.Optional[dict[str, t.Any]] = None,
    ):
        """Initialize the client for project and endpoint using a token.

        Parameters
        ----------
        project_id : str
            ID of the project to create a client for.
        delete_polarion_work_items : bool
            Flag indicating whether to delete work items or set a status.
        polarion_api_endpoint : str
            The URL of the Polarion API endpoint.
        polarion_access_token : str
            A personal access token to access the API.
        custom_work_item : default dm.WorkItem
            Custom WorkItem class with additional attributes.
        batch_size : int, default 100
            Maximum amount of items created in one POST request.
        page_size : int, default 100
            Default size of a page when getting items from the API.
        add_work_item_checksum : bool, default False
            Flag whether post WorkItem checksums.
        max_content_size : int, default 2 * 1024**2
            Maximum content-length of the API (default: 2MB).
        httpx_args: t.Optional[dict[str, t.Any]], default None
            Additional parameters, which will be passed to the httpx client.
        """
        warnings.warn(
            f"{self.__class__.__name__} will be deprecated.",
            DeprecationWarning,
            stacklevel=2,
        )

        polarion_client = client.PolarionClient(
            polarion_api_endpoint,
            polarion_access_token,
            httpx_args,
            batch_size,
            page_size,
            max_content_size,
        )
        self.project_client = polarion_client.generate_project_client(
            project_id,
            None if delete_polarion_work_items else "deleted",
            add_work_item_checksum,
        )
        self.project_id = project_id
        self.custom_work_item = custom_work_item

    def project_exists(self) -> bool:
        """Return True if self.project_id exists and False if not."""
        return self.project_client.exists()

    def delete_work_item_attachment(
        self, work_item_attachment: dm.WorkItemAttachment
    ):
        """Delete the given work item attachment."""
        self.project_client.work_items.attachments.delete(work_item_attachment)

    def update_work_item_attachment(
        self, work_item_attachment: dm.WorkItemAttachment
    ):
        """Update the given work item attachment in Polarion."""
        self.project_client.work_items.attachments.update(work_item_attachment)

    def create_work_item_attachments(
        self,
        work_item_attachments: list[dm.WorkItemAttachment],
    ):
        """Create the given work item attachment in Polarion."""
        self.project_client.work_items.attachments.create(
            work_item_attachments
        )

    def get_work_item(
        self,
        work_item_id: str,
    ) -> client.WorkItemType | None:
        """Return one specific work item with all fields.

        This also includes all linked work items and attachments. If
        there are to many of these to get them in one request, the
        truncated flags for linked_work_items and attachments will be
        set to True.
        """
        return self.project_client.work_items.get(
            work_item_id, self.custom_work_item
        )

    def get_document(
        self,
        space_id: str,
        document_name: str,
        fields: dict[str, str] | None = None,
        include: str | None = None,
        revision: str | None = None,
    ) -> dm.Document | None:
        """Return the document with the given document_name and space_id."""
        return self.project_client.documents.get(
            space_id,
            document_name,
            fields=fields,
            include=include,
            revision=revision,
        )

    def create_work_items(self, work_items: list[client.WorkItemType]):
        """Create the given list of work items."""
        self.project_client.work_items.create(work_items)

    def update_work_item(self, work_item: client.WorkItemType):
        """Update the given work item in Polarion.

        Only fields not set to None will be updated in Polarion. None
        fields will stay untouched.
        """
        self.project_client.work_items.update(work_item)

    def get_work_item_links(
        self,
        work_item_id: str,
        fields: dict[str, str] | None = None,
        include: str | None = None,
        page_size: int = 100,
        page_number: int = 1,
    ) -> tuple[list[dm.WorkItemLink], bool]:
        """Get the work item links for the given work item on a page.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        return self.project_client.work_items.links.get_multi(
            work_item_id,
            page_size=page_size,
            page_number=page_number,
            fields=fields,
            include=include,
        )

    def get_test_records(
        self,
        test_run_id: str,
        fields: dict[str, str] | None = None,
        page_size: int = 100,
        page_number: int = 1,
    ) -> tuple[list[dm.TestRecord], bool]:
        """Return the test records on a defined page matching the given query.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        return self.project_client.test_runs.records.get_multi(
            test_run_id,
            fields=fields,
            page_size=page_size,
            page_number=page_number,
        )

    def get_test_runs(
        self,
        query: str = "",
        fields: dict[str, str] | None = None,
        page_size: int = 100,
        page_number: int = 1,
    ) -> tuple[list[dm.TestRun], bool]:
        """Return the test runs on a defined page matching the given query.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        return self.project_client.test_runs.get_multi(
            query, fields=fields, page_size=page_size, page_number=page_number
        )

    def create_test_runs(self, test_runs: list[dm.TestRun]):
        """Create the given list of test runs."""
        self.project_client.test_runs.create(test_runs)

    def update_test_run(self, test_run: dm.TestRun):
        """Create the given list of test runs."""
        self.project_client.test_runs.update(test_run)

    def create_test_records(
        self,
        test_run_id: str,
        test_records: list[dm.TestRecord],
    ):
        """Create the given list of test records."""
        for rec in test_records:
            rec.test_run_id = test_run_id
        self.project_client.test_runs.records.create(test_records)

    def update_test_record(self, test_run_id: str, test_record: dm.TestRecord):
        """Create the given list of test records."""
        test_record.test_run_id = test_run_id
        self.project_client.test_runs.records.update(test_record)

    def get_all_work_item_attachments(
        self, work_item_id: str, fields: dict[str, str] | None = None
    ) -> list[dm.WorkItemAttachment]:
        """Get all work item attachments for a given work item.

        Will handle pagination automatically. Define a fields dictionary
        as described in the Polarion API documentation to get certain
        fields.
        """
        return self.project_client.work_items.attachments.get_all(
            work_item_id, fields=fields
        )

    def create_work_item_attachment(
        self, work_item_attachment: dm.WorkItemAttachment
    ):
        """Update the given work item attachment in Polarion."""
        self.project_client.work_items.attachments.create(work_item_attachment)

    def get_all_work_items(
        self, query: str = "", fields: dict[str, str] | None = None
    ) -> list[WorkItemType]:
        """Get all work items matching the given query.

        Will handle pagination automatically. Define a fields dictionary
        as described in the Polarion API documentation to get certain
        fields.
        """
        return self.project_client.work_items.get_all(
            query, fields=fields, work_item_cls=self.custom_work_item
        )

    def create_work_item(self, work_item: WorkItemType):
        """Create a single given work item."""
        self.create_work_items([work_item])

    def delete_work_items(self, work_item_ids: list[str]):
        """Delete or mark the defined work items as deleted."""
        self.project_client.work_items.delete(
            [dm.WorkItem(wid) for wid in work_item_ids]
        )

    def delete_work_item(self, work_item_id: str):
        """Delete or mark the defined work item as deleted."""
        return self.delete_work_items([work_item_id])

    def get_all_work_item_links(
        self,
        work_item_id: str,
        fields: dict[str, str] | None = None,
        include: str | None = None,
    ) -> list[dm.WorkItemLink]:
        """Get all work item links for the given work item.

        Define a fields dictionary as described in the Polarion API
        documentation to get certain fields.
        """
        return self.project_client.work_items.links.get_all(
            work_item_id=work_item_id,
            fields=fields,
            include=include,
        )

    def create_work_item_links(self, work_item_links: list[dm.WorkItemLink]):
        """Create the links between the work items in work_item_links."""
        self.project_client.work_items.links.create(work_item_links)

    def create_work_item_link(self, work_item_link: dm.WorkItemLink):
        """Create the link between the work items in work_item_link."""
        self.create_work_item_links([work_item_link])

    def delete_work_item_links(self, work_item_links: list[dm.WorkItemLink]):
        """Delete the links between the work items in work_item_link."""
        self.project_client.work_items.links.delete(work_item_links)

    def delete_work_item_link(self, work_item_link: dm.WorkItemLink):
        """Delete the links between the work items in work_item_link."""
        self.delete_work_item_links([work_item_link])

    def get_all_test_runs(
        self,
        query: str = "",
        fields: dict[str, str] | None = None,
    ) -> list[dm.TestRun]:
        """Get all test runs matching the given query.

        Will handle pagination automatically. Define a fields dictionary
        as described in the Polarion API documentation to get certain
        fields.
        """
        return self.project_client.test_runs.get_all(query, fields=fields)

    def get_all_test_records(
        self,
        test_run_id: str,
        fields: dict[str, str] | None = None,
    ) -> list[dm.TestRecord]:
        """Get all test records matching the given query.

        Will handle pagination automatically. Define a fields dictionary
        as described in the Polarion API documentation to get certain
        fields.
        """
        return self.project_client.test_runs.records.get_all(
            test_run_id, fields=fields
        )

    def create_test_run(self, test_run: dm.TestRun):
        """Create the given test run."""
        self.create_test_runs([test_run])

    def create_test_record(self, test_run_id: str, test_record: dm.TestRecord):
        """Create the given list of test records."""
        self.create_test_records(test_run_id, [test_record])



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitem_approvals_list_post_request_data_item_relationships_user import (
        WorkitemApprovalsListPostRequestDataItemRelationshipsUser,
    )


T = TypeVar("T", bound="WorkitemApprovalsListPostRequestDataItemRelationships")


@_attrs_define
class WorkitemApprovalsListPostRequestDataItemRelationships:
    """
    Attributes:
        user (Union[Unset, WorkitemApprovalsListPostRequestDataItemRelationshipsUser]):
    """

    user: Union[
        Unset, "WorkitemApprovalsListPostRequestDataItemRelationshipsUser"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitem_approvals_list_post_request_data_item_relationships_user import (
            WorkitemApprovalsListPostRequestDataItemRelationshipsUser,
        )

        d = src_dict.copy()
        _user = d.pop("user", UNSET)
        user: Union[
            Unset, WorkitemApprovalsListPostRequestDataItemRelationshipsUser
        ]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = WorkitemApprovalsListPostRequestDataItemRelationshipsUser.from_dict(
                _user
            )

        workitem_approvals_list_post_request_data_item_relationships_obj = cls(
            user=user,
        )

        workitem_approvals_list_post_request_data_item_relationships_obj.additional_properties = (
            d
        )
        return workitem_approvals_list_post_request_data_item_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Implementation of the documents client."""
import urllib.parse

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.documents import get_document

from . import base_classes as bc


class Documents(bc.UpdatableItemsClient[dm.Document]):
    """A client to work with documents in Polarion."""

    def get(
        self,
        space_id: str,
        document_name: str,
        fields: dict[str, str] | None = None,
        include: str | None | oa_types.Unset = None,
        revision: str | None | oa_types.Unset = None,
    ) -> dm.Document | None:
        """Return the document with the given document_name and space_id."""
        if include is None:
            include = oa_types.UNSET

        if revision is None:
            revision = oa_types.UNSET

        if " " in space_id or " " in document_name:
            space_id = urllib.parse.quote(
                space_id, safe="/", encoding=None, errors=None
            )
            document_name = urllib.parse.quote(
                document_name, safe="/", encoding=None, errors=None
            )
        if fields is None:
            fields = self._client.default_fields.documents

        sparse_fields = self._build_sparse_fields(fields)
        response = get_document.sync_detailed(
            self._project_id,
            space_id,
            document_name,
            client=self._client.client,
            fields=sparse_fields,
            include=include,
            revision=revision,
        )

        self._raise_on_error(response)

        document_response = response.parsed

        if isinstance(
            document_response, api_models.DocumentsSingleGetResponse
        ) and (data := document_response.data):
            if not getattr(data.meta, "errors", []):
                assert (attributes := data.attributes)
                assert isinstance(data.id, str)
                home_page_content = self._handle_text_content(
                    attributes.home_page_content
                )

                return dm.Document(
                    id=data.id,
                    module_folder=self.unset_to_none(attributes.module_folder),
                    module_name=self.unset_to_none(attributes.module_name),
                    type=self.unset_to_none(attributes.type),
                    status=self.unset_to_none(attributes.status),
                    home_page_content=home_page_content,
                )
        return None

    def _update(self, to_update: dm.Document | list[dm.Document]):
        raise NotImplementedError

    def get_multi(
        self, *args, page_size=100, page_number=1, **kwargs
    ) -> tuple[list[dm.Document], bool]:
        """Return a list of documents - Not implemented yet."""
        raise NotImplementedError

    def _create(self, items: dm.Document | list[dm.Document]):
        raise NotImplementedError

    def _delete(self, items: dm.Document | list[dm.Document]):
        raise NotImplementedError



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class FeatureselectionsSingleGetResponseDataAttributesSelectionType(str, Enum):
    EXCLUDED = "excluded"
    IMPLICITLY_INCLUDED = "implicitly-included"
    INCLUDED = "included"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import json

import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import TEST_DOCUMENT_RESPONSE


def test_get_document_with_all_fields(
    client: polarion_api.OpenAPIPolarionProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_DOCUMENT_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    document = client.get_document(
        "MySpaceId", "MyDocumentName", {"fields[documents]": "@all"}
    )

    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert isinstance(document, polarion_api.data_models.Document)
    assert len(reqs) == 1
    assert document == polarion_api.data_models.Document(
        "MyProjectId/MySpaceId/MyDocumentName",
        "MySpaceId",
        "MyDocumentName",
        "standardSpecification",
        "open",
        polarion_api.data_models.TextContent(
            type="text/html", value="<h1>My text value</h1>"
        ),
    )
